@@ -10,12 +10,13 @@ instead, one should use the factory function `get_file`.
 """
 
 import io
-import math
 import os
-import platform
 import re
 import sys
+import math
+import pathlib
 import tempfile
+import platform
 from distutils.version import LooseVersion
 
 from os import SEEK_SET, SEEK_CUR, SEEK_END
@@ -182,7 +183,7 @@ def relative_uri(source, target):
     return relative
 
 
-class _TruncatedReader(object):
+class _TruncatedReader:
     """
     Reads until a given delimiter is found.  Only works with
     RandomAccessFile and InputStream, though as this is a private
@@ -257,7 +258,7 @@ class _TruncatedReader(object):
 
 
 @six.add_metaclass(util.InheritDocstrings)
-class GenericFile(object):
+class GenericFile:
     """
     Base class for an abstraction layer around a number of different
     file-like types.  Each of its subclasses handles a particular kind
@@ -376,7 +377,7 @@ class GenericFile(object):
     """
 
     def write_array(self, array):
-        _array_tofile(None, self.write, np.ascontiguousarray(array))
+        _array_tofile(None, self.write, array.ravel(order='A'))
 
     def seek(self, offset, whence=0):
         """
@@ -656,7 +657,7 @@ class GenericFile(object):
         return np.frombuffer(buff, np.uint8, size, 0)
 
 
-class GenericWrapper(object):
+class GenericWrapper:
     """
     A wrapper around a `GenericFile` object so that closing only
     happens in the very outer layer.
@@ -755,7 +756,7 @@ class RealFile(RandomAccessFile):
             arr.flush()
             self.fast_forward(len(arr.data))
         else:
-            _array_tofile(self._fd, self._fd.write, np.ascontiguousarray(arr))
+            _array_tofile(self._fd, self._fd.write, arr.ravel(order='A'))
 
     def can_memmap(self):
         return True
@@ -1104,7 +1105,7 @@ def get_uri(file_obj):
     return getattr(file_obj, 'name', '')
 
 
-def get_file(init, mode='r', uri=None):
+def get_file(init, mode='r', uri=None, close=False):
     """
     Returns a `GenericFile` instance suitable for wrapping the given
     object `init`.
@@ -1146,6 +1147,10 @@ def get_file(init, mode='r', uri=None):
         `init` refers to a regular filesystem file.  It is not required
         if URI resolution is not used in the file.
 
+    close : bool
+        If ``True``, closes the underlying file handle when this object is
+        closed. Defaults to ``False``.
+
     Returns
     -------
     fd : GenericFile
@@ -1167,9 +1172,9 @@ def get_file(init, mode='r', uri=None):
                     init.mode, mode))
         return GenericWrapper(init)
 
-    elif isinstance(init, str):
-        parsed = urlparse.urlparse(init)
-        if parsed.scheme == 'http':
+    elif isinstance(init, (str, pathlib.Path)):
+        parsed = urlparse.urlparse(str(init))
+        if parsed.scheme in ['http', 'https']:
             if 'w' in mode:
                 raise ValueError(
                     "HTTP connections can not be opened for writing")
@@ -1179,7 +1184,14 @@ def get_file(init, mode='r', uri=None):
                 realmode = 'r+b'
             else:
                 realmode = mode + 'b'
-            realpath = url2pathname(parsed.path)
+            # Windows paths are not URIs, and so they should not be parsed as
+            # such. Otherwise, the drive component of the path can get lost.
+            # This is not an ideal solution, but we can't use pathlib here
+            # because it doesn't handle URIs properly.
+            if sys.platform.startswith('win') and parsed.scheme in string.ascii_letters:
+                realpath = str(init)
+            else:
+                realpath = url2pathname(parsed.path)
             if mode == 'w':
                 fd = atomicfile.atomic_open(realpath, realmode)
             else:
@@ -1209,16 +1221,16 @@ def get_file(init, mode='r', uri=None):
             else:
                 init2 = init
             if isinstance(init2, io.RawIOBase):
-                result = RealFile(init2, mode, uri=uri)
+                result = RealFile(init2, mode, uri=uri, close=close)
             else:
                 result = MemoryIO(init2, mode, uri=uri)
             result._secondary_fd = init
             return result
         else:
             if mode == 'w':
-                return OutputStream(init, uri=uri)
+                return OutputStream(init, uri=uri, close=close)
             elif mode == 'r':
-                return InputStream(init, mode, uri=uri)
+                return InputStream(init, mode, uri=uri, close=close)
             else:
                 raise ValueError(
                     "File '{0}' could not be opened in 'rw' mode".format(init))
@@ -1243,10 +1255,10 @@ def get_file(init, mode='r', uri=None):
         return MemoryIO(init, mode, uri=uri)
 
     elif mode == 'w' and hasattr(init, 'write'):
-        return OutputStream(init, uri=uri)
+        return OutputStream(init, uri=uri, close=close)
 
     elif mode == 'r' and hasattr(init, 'read'):
-        return InputStream(init, mode, uri=uri)
+        return InputStream(init, mode, uri=uri, close=close)
 
     raise ValueError("Can't handle '{0}' as a file for mode '{1}'".format(
         init, mode))
