@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 
-from ...asdftypes import AsdfType
+from ...types import AsdfType
 from ...yamlutil import custom_tree_to_tagged_tree
 
 
@@ -44,4 +44,5 @@ class ExtensionMetadata(AsdfType):
 from .constant import ConstantType
 from .ndarray import NDArrayType
 from .complex import ComplexType
+from .integer import IntegerType
 from .external_reference import ExternalArrayReference
