@@ -8,7 +8,7 @@ from numpy import ma
 
 from jsonschema import ValidationError
 
-from ...asdftypes import AsdfType
+from ...types import AsdfType
 from ... import schema
 from ... import util
 from ... import yamlutil
@@ -240,6 +240,8 @@ class NDArrayType(AsdfType):
         self._offset = offset
         self._strides = strides
         self._order = order
+        if not asdffile.blocks.lazy_load:
+            self._make_array()
 
     def _make_array(self):
         if self._array is None:
@@ -250,6 +252,8 @@ class NDArrayType(AsdfType):
                 shape, self._dtype, block.data,
                 self._offset, self._strides, self._order)
             self._array = self._apply_mask(self._array, self._mask)
+            if block.readonly:
+                self._array.setflags(write=False)
         return self._array
 
     def _apply_mask(self, array, mask):
@@ -342,6 +346,16 @@ class NDArrayType(AsdfType):
             raise AttributeError()
         return getattr(self._make_array(), attr)
 
+    def __setitem__(self, *args):
+        # This workaround appears to be necessary in order to avoid a segfault
+        # in the case that array assignment causes an exception. The segfault
+        # originates from the call to __repr__ inside the traceback report.
+        try:
+            self._make_array().__setitem__(*args)
+        except Exception as e:
+            self._array = None
+            raise e from None
+
     @classmethod
     def from_tree(cls, node, ctx):
         if isinstance(node, list):
@@ -370,7 +384,7 @@ class NDArrayType(AsdfType):
             strides = node.get('strides', None)
             mask = node.get('mask', None)
 
-            return cls(source, shape, dtype, offset, strides, 'C', mask, ctx)
+            return cls(source, shape, dtype, offset, strides, 'A', mask, ctx)
 
         raise TypeError("Invalid ndarray description.")
 
@@ -507,7 +521,7 @@ for op in [
         '__imul__', '__idiv__', '__itruediv__', '__ifloordiv__',
         '__imod__', '__ipow__', '__ilshift__', '__irshift__',
         '__iand__', '__ixor__', '__ior__', '__getitem__',
-        '__delitem__', '__contains__', '__setitem__']:
+        '__delitem__', '__contains__']:
     setattr(NDArrayType, op, _make_operation(op))
 
 
