@@ -5,10 +5,11 @@
 import io
 import os
 import sys
+
 import pytest
 
 import asdf
-from asdf import asdftypes
+from asdf import types
 from asdf import extension
 from asdf import util
 from asdf import versioning
@@ -22,7 +23,7 @@ TEST_DATA_PATH = str(helpers.get_test_data_path(''))
 def test_custom_tag():
     import fractions
 
-    class FractionType(asdftypes.AsdfType):
+    class FractionType(types.CustomType):
         name = 'fraction'
         organization = 'nowhere.org'
         version = (1, 0, 0)
@@ -37,7 +38,7 @@ def test_custom_tag():
         def from_tree(cls, tree, ctx):
             return fractions.Fraction(tree[0], tree[1])
 
-    class FractionExtension(object):
+    class FractionExtension:
         @property
         def types(self):
             return [FractionType]
@@ -70,16 +71,14 @@ b: !core/complex-1.0.0
     """
 
     buff = helpers.yaml_to_asdf(yaml)
-    with asdf.AsdfFile.open(
-        buff, extensions=FractionExtension()) as ff:
+    with asdf.open(buff, extensions=FractionExtension()) as ff:
         assert ff.tree['a'] == fractions.Fraction(2, 3)
 
         buff = io.BytesIO()
         ff.write_to(buff)
 
     buff = helpers.yaml_to_asdf(yaml)
-    with asdf.AsdfFile.open(
-            buff, extensions=FractionCallable()) as ff:
+    with asdf.open(buff, extensions=FractionCallable()) as ff:
         assert ff.tree['a'] == fractions.Fraction(2, 3)
 
         buff = io.BytesIO()
@@ -95,7 +94,7 @@ a: !core/complex-42.0.0
 
     buff = helpers.yaml_to_asdf(yaml)
     with pytest.warns(None) as warning:
-        with asdf.AsdfFile.open(buff, ignore_version_mismatch=False) as ff:
+        with asdf.open(buff, ignore_version_mismatch=False) as ff:
             assert isinstance(ff.tree['a'], complex)
 
     assert len(warning) == 1
@@ -106,7 +105,7 @@ a: !core/complex-42.0.0
     # Make sure warning is repeatable
     buff.seek(0)
     with pytest.warns(None) as warning:
-        with asdf.AsdfFile.open(buff, ignore_version_mismatch=False) as ff:
+        with asdf.open(buff, ignore_version_mismatch=False) as ff:
             assert isinstance(ff.tree['a'], complex)
 
     assert len(warning) == 1
@@ -117,7 +116,7 @@ a: !core/complex-42.0.0
     # Make sure the warning does not occur if it is being ignored (default)
     buff.seek(0)
     with pytest.warns(None) as warning:
-        with asdf.AsdfFile.open(buff) as ff:
+        with asdf.open(buff) as ff:
             assert isinstance(ff.tree['a'], complex)
 
     assert len(warning) == 0, helpers.display_warnings(warning)
@@ -131,7 +130,7 @@ a: !core/complex-1.0.1
 
     buff = helpers.yaml_to_asdf(yaml)
     with pytest.warns(None) as warning:
-        with asdf.AsdfFile.open(buff, ignore_version_mismatch=False) as ff:
+        with asdf.open(buff, ignore_version_mismatch=False) as ff:
             assert isinstance(ff.tree['a'], complex)
 
     assert len(warning) == 0
@@ -151,7 +150,7 @@ a: !core/complex-42.0.0
         handle.write(buff.read())
 
     with pytest.warns(None) as w:
-        with asdf.AsdfFile.open(testfile, ignore_version_mismatch=False) as ff:
+        with asdf.open(testfile, ignore_version_mismatch=False) as ff:
             assert ff._fname == "file://{}".format(testfile)
             assert isinstance(ff.tree['a'], complex)
 
@@ -165,7 +164,7 @@ def test_version_mismatch_with_supported_versions():
     """Make sure that defining the supported_versions field does not affect
     whether or not schema mismatch warnings are triggered."""
 
-    class CustomFlow(object):
+    class CustomFlow:
         pass
 
     class CustomFlowType(CustomTestType):
@@ -176,7 +175,7 @@ def test_version_mismatch_with_supported_versions():
         standard = 'custom'
         types = [CustomFlow]
 
-    class CustomFlowExtension(object):
+    class CustomFlowExtension:
         @property
         def types(self):
             return [CustomFlowType]
@@ -200,7 +199,7 @@ flow_thing:
 """
     buff = helpers.yaml_to_asdf(yaml)
     with pytest.warns(None) as w:
-        data = asdf.AsdfFile.open(
+        data = asdf.open(
             buff, ignore_version_mismatch=False,
             extensions=CustomFlowExtension())
     assert len(w) == 1, helpers.display_warnings(w)
@@ -209,30 +208,46 @@ flow_thing:
         "file, but latest supported version is 1.1.0")
 
 
-def test_versioned_writing():
+def test_versioned_writing(monkeypatch):
     from ..tags.core.complex import ComplexType
 
     # Create a bogus version map
-    versioning._version_map['42.0.0'] = {
+    monkeypatch.setitem(versioning._version_map, '42.0.0', {
         'FILE_FORMAT': '42.0.0',
         'YAML_VERSION': '1.1',
         'tags': {
             'tag:stsci.edu:asdf/core/complex': '42.0.0',
             'tag:stscu.edu:asdf/core/asdf': '1.0.0'
-        }
-    }
+        },
+        # We need to insert these explicitly since we're monkeypatching
+        'core': {
+            'tag:stsci.edu:asdf/core/complex': '42.0.0',
+            'tag:stscu.edu:asdf/core/asdf': '1.0.0'
+        },
+        'standard': {}
+    })
 
-    versioning.supported_versions.append(versioning.AsdfVersion('42.0.0'))
+    # Add bogus version to supported versions
+    monkeypatch.setattr(versioning, 'supported_versions',
+        versioning.supported_versions + [versioning.AsdfVersion('42.0.0')]
+    )
 
-    class FancyComplexType(ComplexType, asdftypes.CustomType):
+    class FancyComplexType(types.CustomType):
+        name = 'core/complex'
+        organization = 'stsci.edu'
+        standard = 'asdf'
         version = (42, 0, 0)
+        types = [complex]
 
-    # This is a sanity check to ensure that the custom FancyComplexType does
-    # not get added to ASDF's built-in extension, since this would cause any
-    # subsequent tests that rely on ComplexType to fail.
-    assert not issubclass(FancyComplexType, asdftypes.AsdfTypeMeta)
+        @classmethod
+        def to_tree(cls, node, ctx):
+            return ComplexType.to_tree(node, ctx)
 
-    class FancyComplexExtension(object):
+        @classmethod
+        def from_tree(cls, tree, ctx):
+            return ComplexType.from_tree(tree, ctx)
+
+    class FancyComplexExtension:
         @property
         def types(self):
             return [FancyComplexType]
@@ -256,12 +271,9 @@ def test_versioned_writing():
 
     assert b'complex-42.0.0' in buff.getvalue()
 
-    del versioning._version_map['42.0.0']
-    versioning.supported_versions.pop()
-
 
 def test_longest_match():
-    class FancyComplexExtension(object):
+    class FancyComplexExtension:
         @property
         def types(self):
             return []
@@ -285,15 +297,15 @@ def test_longest_match():
 
 
 def test_module_versioning():
-    class NoModuleType(asdftypes.AsdfType):
+    class NoModuleType(types.CustomType):
         # It seems highly unlikely that this would be a real module
         requires = ['qkjvqdja']
 
-    class HasCorrectPytest(asdftypes.AsdfType):
+    class HasCorrectPytest(types.CustomType):
         # This means it requires 1.0.0 or greater, so it should succeed
         requires = ['pytest-1.0.0']
 
-    class DoesntHaveCorrectPytest(asdftypes.AsdfType):
+    class DoesntHaveCorrectPytest(types.CustomType):
         requires = ['pytest-91984.1.7']
 
     nmt = NoModuleType()
@@ -325,7 +337,7 @@ undefined_data:
 """
     buff = helpers.yaml_to_asdf(yaml)
     with pytest.warns(None) as warning:
-        afile = asdf.AsdfFile.open(buff)
+        afile = asdf.open(buff)
         missing = afile.tree['undefined_data']
 
     assert missing[0] == 5
@@ -344,7 +356,7 @@ undefined_data:
     # Make sure no warning occurs if explicitly ignored
     buff.seek(0)
     with pytest.warns(None) as warning:
-        afile = asdf.AsdfFile.open(buff, ignore_unrecognized_tag=True)
+        afile = asdf.open(buff, ignore_unrecognized_tag=True)
     assert len(warning) == 0
 
 
@@ -356,12 +368,12 @@ def test_newer_tag():
     # fairly contrived but we want to test whether ASDF can handle backwards
     # compatibility even when an explicit tag class for different versions of a
     # schema is not available.
-    class CustomFlow(object):
+    class CustomFlow:
         def __init__(self, c=None, d=None):
             self.c = c
             self.d = d
 
-    class CustomFlowType(asdftypes.CustomType):
+    class CustomFlowType(types.CustomType):
         version = '1.1.0'
         name = 'custom_flow'
         organization = 'nowhere.org'
@@ -379,7 +391,7 @@ def test_newer_tag():
         def to_tree(cls, data, ctx):
             tree = dict(c=data.c, d=data.d)
 
-    class CustomFlowExtension(object):
+    class CustomFlowExtension:
         @property
         def types(self):
             return [CustomFlowType]
@@ -402,7 +414,7 @@ flow_thing:
     d: 3.14
 """
     new_buff = helpers.yaml_to_asdf(new_yaml)
-    new_data = asdf.AsdfFile.open(new_buff, extensions=CustomFlowExtension())
+    new_data = asdf.open(new_buff, extensions=CustomFlowExtension())
     assert type(new_data.tree['flow_thing']) == CustomFlow
 
     old_yaml = """
@@ -413,7 +425,7 @@ flow_thing:
 """
     old_buff = helpers.yaml_to_asdf(old_yaml)
     with pytest.warns(None) as warning:
-        asdf.AsdfFile.open(old_buff, extensions=CustomFlowExtension())
+        asdf.open(old_buff, extensions=CustomFlowExtension())
 
     assert len(warning) == 1, helpers.display_warnings(warning)
     # We expect this warning since it will not be possible to convert version
@@ -423,26 +435,26 @@ flow_thing:
         "tag:nowhere.org:custom/custom_flow-1.0.0 to custom type")
 
 def test_incompatible_version_check():
-    class TestType0(asdftypes.CustomType):
+    class TestType0(types.CustomType):
         supported_versions = versioning.AsdfSpec('>=1.2.0')
 
     assert TestType0.incompatible_version('1.1.0') == True
     assert TestType0.incompatible_version('1.2.0') == False
     assert TestType0.incompatible_version('2.0.1') == False
 
-    class TestType1(asdftypes.CustomType):
+    class TestType1(types.CustomType):
         supported_versions = versioning.AsdfVersion('1.0.0')
 
     assert TestType1.incompatible_version('1.0.0') == False
     assert TestType1.incompatible_version('1.1.0') == True
 
-    class TestType2(asdftypes.CustomType):
+    class TestType2(types.CustomType):
         supported_versions = '1.0.0'
 
     assert TestType2.incompatible_version('1.0.0') == False
     assert TestType2.incompatible_version('1.1.0') == True
 
-    class TestType3(asdftypes.CustomType):
+    class TestType3(types.CustomType):
         # This doesn't make much sense, but it's just for the sake of example
         supported_versions = ['1.0.0', versioning.AsdfSpec('>=2.0.0')]
 
@@ -451,7 +463,7 @@ def test_incompatible_version_check():
     assert TestType3.incompatible_version('2.0.0') == False
     assert TestType3.incompatible_version('2.0.1') == False
 
-    class TestType4(asdftypes.CustomType):
+    class TestType4(types.CustomType):
         supported_versions = ['1.0.0', versioning.AsdfVersion('1.1.0')]
 
     assert TestType4.incompatible_version('1.0.0') == False
@@ -459,7 +471,7 @@ def test_incompatible_version_check():
     assert TestType4.incompatible_version('1.1.0') == False
     assert TestType4.incompatible_version('1.1.1') == True
 
-    class TestType5(asdftypes.CustomType):
+    class TestType5(types.CustomType):
         supported_versions = \
             [versioning.AsdfSpec('<1.0.0'), versioning.AsdfSpec('>=2.0.0')]
 
@@ -470,19 +482,19 @@ def test_incompatible_version_check():
     assert TestType5.incompatible_version('1.1.0') == True
 
     with pytest.raises(ValueError):
-        class TestType6(asdftypes.CustomType):
+        class TestType6(types.CustomType):
             supported_versions = 'blue'
     with pytest.raises(ValueError):
-        class TestType6(asdftypes.CustomType):
+        class TestType6(types.CustomType):
             supported_versions = ['1.1.0', '2.2.0', 'blue']
 
 def test_supported_versions():
-    class CustomFlow(object):
+    class CustomFlow:
         def __init__(self, c=None, d=None):
             self.c = c
             self.d = d
 
-    class CustomFlowType(asdftypes.CustomType):
+    class CustomFlowType(types.CustomType):
         version = '1.1.0'
         supported_versions = [(1,0,0), versioning.AsdfSpec('>=1.1.0')]
         name = 'custom_flow'
@@ -506,7 +518,7 @@ def test_supported_versions():
             else:
                 tree = dict(c=data.c, d=data.d)
 
-    class CustomFlowExtension(object):
+    class CustomFlowExtension:
         @property
         def types(self):
             return [CustomFlowType]
@@ -535,18 +547,18 @@ flow_thing:
     b: 3.14
 """
     new_buff = helpers.yaml_to_asdf(new_yaml)
-    new_data = asdf.AsdfFile.open(new_buff, extensions=CustomFlowExtension())
+    new_data = asdf.open(new_buff, extensions=CustomFlowExtension())
     assert type(new_data.tree['flow_thing']) == CustomFlow
 
     old_buff = helpers.yaml_to_asdf(old_yaml)
-    old_data = asdf.AsdfFile.open(old_buff, extensions=CustomFlowExtension())
+    old_data = asdf.open(old_buff, extensions=CustomFlowExtension())
     assert type(old_data.tree['flow_thing']) == CustomFlow
 
 def test_unsupported_version_warning():
-    class CustomFlow(object):
+    class CustomFlow:
         pass
 
-    class CustomFlowType(asdftypes.CustomType):
+    class CustomFlowType(types.CustomType):
         version = '1.0.0'
         supported_versions = [(1,0,0)]
         name = 'custom_flow'
@@ -554,7 +566,7 @@ def test_unsupported_version_warning():
         standard = 'custom'
         types = [CustomFlow]
 
-    class CustomFlowExtension(object):
+    class CustomFlowExtension:
         @property
         def types(self):
             return [CustomFlowType]
@@ -579,9 +591,115 @@ flow_thing:
     buff = helpers.yaml_to_asdf(yaml)
 
     with pytest.warns(None) as _warnings:
-        data = asdf.AsdfFile.open(buff, extensions=CustomFlowExtension())
+        data = asdf.open(buff, extensions=CustomFlowExtension())
 
     assert len(_warnings) == 1
     assert str(_warnings[0].message) == (
         "Version 1.1.0 of tag:nowhere.org:custom/custom_flow is not compatible "
         "with any existing tag implementations")
+
+def test_extension_override(tmpdir):
+
+    gwcs = pytest.importorskip('gwcs', '0.9.0')
+
+    from asdf.extension import default_extensions
+    default_extensions.reset()
+
+    version = str(versioning.default_version)
+    tmpfile = str(tmpdir.join('override.asdf'))
+
+    with asdf.AsdfFile() as aa:
+        wti = aa.type_index._write_type_indices[version]
+        assert wti.from_custom_type(gwcs.WCS) is gwcs.tags.WCSType
+        aa.tree['wcs'] = gwcs.WCS(output_frame='icrs')
+        aa.write_to(tmpfile)
+
+    with open(tmpfile, 'rb') as ff:
+        contents = str(ff.read())
+        assert gwcs.tags.WCSType.yaml_tag in contents
+
+
+def test_extension_override_subclass(tmpdir):
+
+    gwcs = pytest.importorskip('gwcs', '0.9.0')
+    astropy = pytest.importorskip('astropy', '3.0.0')
+    from astropy.modeling import models
+
+    from asdf.extension import default_extensions
+    default_extensions.reset()
+
+    version = str(versioning.default_version)
+    tmpfile = str(tmpdir.join('override.asdf'))
+
+    class SubclassWCS(gwcs.WCS):
+        pass
+
+    with asdf.AsdfFile() as aa:
+        wti = aa.type_index._write_type_indices[version]
+        assert wti.from_custom_type(gwcs.WCS) is gwcs.tags.WCSType
+        assert wti.from_custom_type(SubclassWCS) is gwcs.tags.WCSType
+        # The duplication here is deliberate: make sure that nothing has changed
+        assert wti.from_custom_type(gwcs.WCS) is gwcs.tags.WCSType
+        aa.tree['wcs'] = SubclassWCS(output_frame='icrs')
+        aa.write_to(tmpfile)
+
+    with open(tmpfile, 'rb') as ff:
+        contents = str(ff.read())
+        assert gwcs.tags.WCSType.yaml_tag in contents
+
+
+def test_tag_without_schema(tmpdir):
+
+    tmpfile = str(tmpdir.join('foo.asdf'))
+
+    class FooType(types.CustomType):
+        name = 'foo'
+
+        def __init__(self, a, b):
+            self.a = a
+            self.b = b
+
+        @classmethod
+        def from_tree(cls, tree, ctx):
+            return cls(tree['a'], tree['b'])
+
+        @classmethod
+        def to_tree(cls, node, ctx):
+            return dict(a=node.a, b=node.b)
+
+        def __eq__(self, other):
+            return self.a == other.a and self.b == other.b
+
+    class FooExtension:
+        @property
+        def types(self):
+            return [FooType]
+
+        @property
+        def tag_mapping(self):
+            return []
+
+        @property
+        def url_mapping(self):
+            return []
+
+    foo = FooType('hello', 42)
+    tree = dict(foo=foo)
+
+    with pytest.warns(UserWarning) as w:
+        with asdf.AsdfFile(tree, extensions=FooExtension()) as af:
+            af.write_to(tmpfile)
+        # There are three validation passes when writing. Eventually this may
+        # change
+        assert len(w) == 3, helpers.display_warnings(w)
+        assert str(w[0].message).startswith('Unable to locate schema file')
+        assert str(w[1].message).startswith('Unable to locate schema file')
+        assert str(w[2].message).startswith('Unable to locate schema file')
+
+    with pytest.warns(UserWarning) as w:
+        with asdf.AsdfFile(tree, extensions=FooExtension()) as ff:
+            assert isinstance(ff.tree['foo'], FooType)
+            assert ff.tree['foo'] == tree['foo']
+        # There is only one validation pass when writing.
+        assert len(w) == 1, helpers.display_warnings(w)
+        assert str(w[0].message).startswith('Unable to locate schema file')
