@@ -15,7 +15,7 @@ import numpy as np
 from numpy.testing import assert_array_equal
 
 import asdf
-from asdf import asdftypes
+from asdf import types
 from asdf import extension
 from asdf import resolver
 from asdf import schema
@@ -46,7 +46,7 @@ class CustomExtension:
                  '/{url_suffix}.yaml')]
 
 
-class TagReferenceType(asdftypes.CustomType):
+class TagReferenceType(types.CustomType):
     """
     This class is used by several tests below for validating foreign type
     references in schemas and ASDF files.
@@ -75,7 +75,7 @@ not_unit:
     from astropy import units as u
 
     buff = helpers.yaml_to_asdf(yaml)
-    with asdf.AsdfFile.open(buff) as ff:
+    with asdf.open(buff) as ff:
         assert isinstance(ff.tree['unit'], u.UnitBase)
         assert not isinstance(ff.tree['not_unit'], u.UnitBase)
         assert isinstance(ff.tree['not_unit'], str)
@@ -167,6 +167,30 @@ required: [foobar]
     schema.check_schema(schema_tree)
 
 
+def test_load_schema_with_file_url(tmpdir):
+    schema_def = """
+%YAML 1.1
+%TAG !asdf! tag:stsci.edu:asdf/
+---
+$schema: "http://stsci.edu/schemas/asdf/asdf-schema-1.0.0"
+id: "http://stsci.edu/schemas/asdf/nugatory/nugatory-1.0.0"
+tag: "tag:stsci.edu:asdf/nugatory/nugatory-1.0.0"
+
+type: object
+properties:
+  foobar:
+      $ref: "{}"
+
+required: [foobar]
+...
+    """.format(resolver.default_resolver('tag:stsci.edu:asdf/core/ndarray-1.0.0'))
+    schema_path = tmpdir.join('nugatory.yaml')
+    schema_path.write(schema_def.encode())
+
+    schema_tree = schema.load_schema(str(schema_path), resolve_references=True)
+    schema.check_schema(schema_tree)
+
+
 def test_schema_caching():
     # Make sure that if we request the same URL, we get the *exact
     # same* object, to ensure the cache is working.
@@ -178,7 +202,7 @@ def test_schema_caching():
 
 
 def test_flow_style():
-    class CustomFlowStyleType(dict, asdftypes.AsdfType):
+    class CustomFlowStyleType(dict, types.CustomType):
         name = 'custom_flow'
         organization = 'nowhere.org'
         version = (1, 0, 0)
@@ -201,7 +225,7 @@ def test_flow_style():
 
 
 def test_style():
-    class CustomStyleType(str, asdftypes.AsdfType):
+    class CustomStyleType(str, types.CustomType):
         name = 'custom_style'
         organization = 'nowhere.org'
         version = (1, 0, 0)
@@ -243,7 +267,7 @@ def test_property_order():
 
 
 def test_invalid_nested():
-    class CustomType(str, asdftypes.AsdfType):
+    class CustomType(str, types.CustomType):
         name = 'custom'
         organization = 'nowhere.org'
         version = (1, 0, 0)
@@ -263,13 +287,13 @@ custom: !<tag:nowhere.org:custom/custom-1.0.0>
     # providing an extension, our custom type will not be recognized and will
     # simply be converted to a raw type.
     with pytest.warns(None) as warning:
-        with asdf.AsdfFile.open(buff):
+        with asdf.open(buff):
             pass
     assert len(warning) == 1
 
     buff.seek(0)
     with pytest.raises(ValidationError):
-        with asdf.AsdfFile.open(buff, extensions=[CustomTypeExtension()]):
+        with asdf.open(buff, extensions=[CustomTypeExtension()]):
             pass
 
     # Make sure tags get validated inside of other tags that know
@@ -282,7 +306,7 @@ array: !core/ndarray-1.0.0
     """
     buff = helpers.yaml_to_asdf(yaml)
     with pytest.raises(ValidationError):
-        with asdf.AsdfFile.open(buff, extensions=[CustomTypeExtension()]):
+        with asdf.open(buff, extensions=[CustomTypeExtension()]):
             pass
 
 
@@ -337,7 +361,7 @@ def test_default_check_in_schema():
 
 
 def test_fill_and_remove_defaults():
-    class DefaultType(dict, asdftypes.AsdfType):
+    class DefaultType(dict, types.CustomType):
         name = 'default'
         organization = 'nowhere.org'
         version = (1, 0, 0)
@@ -353,13 +377,13 @@ custom: !<tag:nowhere.org:custom/default-1.0.0>
   b: {}
     """
     buff = helpers.yaml_to_asdf(yaml)
-    with asdf.AsdfFile.open(buff, extensions=[DefaultTypeExtension()]) as ff:
+    with asdf.open(buff, extensions=[DefaultTypeExtension()]) as ff:
         assert 'a' in ff.tree['custom']
         assert ff.tree['custom']['a'] == 42
         assert ff.tree['custom']['b']['c'] == 82
 
     buff.seek(0)
-    with asdf.AsdfFile.open(buff, extensions=[DefaultTypeExtension()],
+    with asdf.open(buff, extensions=[DefaultTypeExtension()],
                             do_not_fill_defaults=True) as ff:
         assert 'a' not in ff.tree['custom']
         assert 'c' not in ff.tree['custom']['b']
@@ -388,14 +412,14 @@ custom: !<tag:nowhere.org:custom/tag_reference-1.0.0>
     """
 
     buff = helpers.yaml_to_asdf(yaml)
-    with asdf.AsdfFile.open(buff, extensions=[DefaultTypeExtension()]) as ff:
+    with asdf.open(buff, extensions=[DefaultTypeExtension()]) as ff:
         custom = ff.tree['custom']
         assert custom['name'] == "Something"
         assert_array_equal(custom['things'], [1, 2, 3])
 
 
 def test_foreign_tag_reference_validation():
-    class ForeignTagReferenceType(asdftypes.CustomType):
+    class ForeignTagReferenceType(types.CustomType):
         name = 'foreign_tag_reference'
         organization = 'nowhere.org'
         version = (1, 0, 0)
@@ -428,7 +452,7 @@ custom: !<tag:nowhere.org:custom/foreign_tag_reference-1.0.0>
     """
 
     buff = helpers.yaml_to_asdf(yaml)
-    with asdf.AsdfFile.open(buff, extensions=ForeignTypeExtension()) as ff:
+    with asdf.open(buff, extensions=ForeignTypeExtension()) as ff:
         a = ff.tree['custom']['a']
         b = ff.tree['custom']['b']
         assert a['name'] == 'Something'
@@ -447,16 +471,22 @@ def test_self_reference_resolution():
     assert s['anyOf'][1] == s['anyOf'][0]
 
 
-def test_large_literals():
+@pytest.mark.parametrize('use_numpy', [False, True])
+def test_large_literals(use_numpy):
+
+    largeval = 1 << 53
+    if use_numpy:
+        largeval = np.uint64(largeval)
+
     tree = {
-        'large_int': (1 << 53),
+        'large_int': largeval,
     }
 
     with pytest.raises(ValidationError):
         asdf.AsdfFile(tree)
 
     tree = {
-        'large_array': np.array([(1 << 53)], np.uint64)
+        'large_array': np.array([largeval], np.uint64)
     }
 
     ff = asdf.AsdfFile(tree)
@@ -470,10 +500,104 @@ def test_large_literals():
         print(buff.getvalue())
 
 
+def test_read_large_literal():
+
+    value = 1 << 64
+    yaml = """integer: {}""".format(value)
+
+    buff = helpers.yaml_to_asdf(yaml)
+
+    with pytest.warns(UserWarning) as w:
+        with asdf.open(buff) as af:
+            assert af['integer'] == value
+
+        # We get two warnings: one for validation time, and one when defaults
+        # are filled. It seems like we could improve this architecture, though...
+        assert len(w) == 2
+        assert str(w[0].message).startswith('Invalid integer literal value')
+        assert str(w[1].message).startswith('Invalid integer literal value')
+
+
+def test_nested_array():
+    s = {
+        'type': 'object',
+        'properties':  {
+            'stuff': {
+                'type': 'array',
+                'items': {
+                    'type': 'array',
+                    'items': [
+                        { 'type': 'integer' },
+                        { 'type': 'string' },
+                        { 'type': 'number' },
+                    ],
+                    'minItems': 3,
+                    'maxItems': 3
+                }
+            }
+        }
+    }
+
+    good = dict(stuff=[[1, 'hello', 2], [4, 'world', 9.7]])
+    schema.validate(good, schema=s)
+
+    bads = [
+        dict(stuff=[[1, 2, 3]]),
+        dict(stuff=[12,'dldl']),
+        dict(stuff=[[12, 'dldl']]),
+        dict(stuff=[[1, 'hello', 2], [4, 5]]),
+        dict(stuff=[[1, 'hello', 2], [4, 5, 6]])
+    ]
+
+    for b in bads:
+        with pytest.raises(ValidationError):
+            schema.validate(b, schema=s)
+
+
+def test_nested_array_yaml(tmpdir):
+    schema_def = """
+%YAML 1.1
+---
+type: object
+properties:
+  stuff:
+    type: array
+    items:
+      type: array
+      items:
+        - type: integer
+        - type: string
+        - type: number
+      minItems: 3
+      maxItems: 3
+...
+    """
+    schema_path = tmpdir.join('nested.yaml')
+    schema_path.write(schema_def.encode())
+
+    schema_tree = schema.load_schema(str(schema_path))
+    schema.check_schema(schema_tree)
+
+    good = dict(stuff=[[1, 'hello', 2], [4, 'world', 9.7]])
+    schema.validate(good, schema=schema_tree)
+
+    bads = [
+        dict(stuff=[[1, 2, 3]]),
+        dict(stuff=[12,'dldl']),
+        dict(stuff=[[12, 'dldl']]),
+        dict(stuff=[[1, 'hello', 2], [4, 5]]),
+        dict(stuff=[[1, 'hello', 2], [4, 5, 6]])
+    ]
+
+    for b in bads:
+        with pytest.raises(ValidationError):
+            schema.validate(b, schema=schema_tree)
+
+
 @pytest.mark.importorskip('astropy')
 def test_type_missing_dependencies():
 
-    class MissingType(asdftypes.AsdfType):
+    class MissingType(types.CustomType):
         name = 'missing'
         organization = 'nowhere.org'
         version = (1, 1, 0)
@@ -492,7 +616,7 @@ custom: !<tag:nowhere.org:custom/missing-1.1.0>
     """
     buff = helpers.yaml_to_asdf(yaml)
     with pytest.warns(None) as w:
-        with asdf.AsdfFile.open(buff, extensions=[DefaultTypeExtension()]) as ff:
+        with asdf.open(buff, extensions=[DefaultTypeExtension()]) as ff:
             assert ff.tree['custom']['b']['foo'] == 42
 
     assert len(w) == 1
@@ -501,7 +625,7 @@ custom: !<tag:nowhere.org:custom/missing-1.1.0>
 def test_assert_roundtrip_with_extension(tmpdir):
     called_custom_assert_equal = [False]
 
-    class CustomType(dict, asdftypes.AsdfType):
+    class CustomType(dict, types.CustomType):
         name = 'custom_flow'
         organization = 'nowhere.org'
         version = (1, 0, 0)
@@ -617,3 +741,51 @@ def test_custom_validation_with_definitions_bad(tmpdir):
     with pytest.raises(ValidationError):
         with asdf.open(asdf_file, custom_schema=custom_schema_path) as ff:
             pass
+
+
+def test_nonexistent_tag(tmpdir):
+    """
+    This tests the case where a node is tagged with a type that apparently
+    comes from an extension that is known, but the type itself can't be found.
+
+    This could occur when a more recent version of an installed package
+    provides the new type, but an older version of the package is installed.
+    ASDF should still be able to open the file in this case, but it won't be
+    able to restore the type.
+
+    The bug that prompted this test results from attempting to load a schema
+    file that doesn't exist, which is why this test belongs in this file.
+    """
+
+    # This shouldn't ever happen, but it's a useful test case
+    yaml = """
+a: !core/doesnt_exist-1.0.0
+  hello
+    """
+
+    buff = helpers.yaml_to_asdf(yaml)
+    with pytest.warns(None) as w:
+        with asdf.open(buff) as af:
+            assert str(af['a']) == 'hello'
+            # Currently there are 3 warnings since one occurs on each of the
+            # validation passes. It would be good to consolidate these
+            # eventually
+            assert len(w) == 3, helpers.display_warnings(w)
+            assert str(w[0].message).startswith("Unable to locate schema file")
+            assert str(w[1].message).startswith("Unable to locate schema file")
+            assert str(w[2].message).startswith(af['a']._tag)
+
+    # This is a more realistic case since we're using an external extension
+    yaml = """
+a: !<tag:nowhere.org:custom/doesnt_exist-1.0.0>
+  hello
+  """
+
+    buff = helpers.yaml_to_asdf(yaml)
+    with pytest.warns(None) as w:
+        with asdf.open(buff, extensions=CustomExtension()) as af:
+            assert str(af['a']) == 'hello'
+            assert len(w) == 3, helpers.display_warnings(w)
+            assert str(w[0].message).startswith("Unable to locate schema file")
+            assert str(w[1].message).startswith("Unable to locate schema file")
+            assert str(w[2].message).startswith(af['a']._tag)
