@@ -5,9 +5,9 @@
 This module deals with things that change between different versions
 of the ASDF spec.
 """
+from functools import total_ordering
 
 import yaml
-from functools import total_ordering
 
 if getattr(yaml, '__with_libyaml__', None):  # pragma: no cover
     _yaml_base_loader = yaml.CSafeLoader
@@ -19,9 +19,26 @@ from semantic_version import Version, SpecItem, Spec
 from . import generic_io
 from . import resolver
 from . import util
+from .version import version as asdf_version
 
 
-__all__ = ['AsdfVersion', 'AsdfSpec']
+__all__ = ['AsdfVersion', 'AsdfSpec', 'split_tag_version', 'join_tag_version']
+
+
+def split_tag_version(tag):
+    """
+    Split a tag into its base and version.
+    """
+    name, version = tag.rsplit('-', 1)
+    version = AsdfVersion(version)
+    return name, version
+
+
+def join_tag_version(name, version):
+    """
+    Join the root and version of a tag back together.
+    """
+    return '{0}-{1}'.format(name, version)
 
 
 _version_map = {}
@@ -35,16 +52,26 @@ def get_version_map(version):
             with generic_io.get_file(version_map_path, 'r') as fd:
                 version_map = yaml.load(
                     fd, Loader=_yaml_base_loader)
-        except:
+        except Exception:
             raise ValueError(
                 "Could not load version map for version {0}".format(version))
+
+        # Separate the core tags from the rest of the standard for convenience
+        version_map['core'] = {}
+        version_map['standard'] = {}
+        for name, version in version_map['tags'].items():
+            if name.startswith('tag:stsci.edu:asdf/core'):
+                version_map['core'][name] = version
+            else:
+                version_map['standard'][name] = version
+
         _version_map[version] = version_map
 
     return version_map
 
 
 @total_ordering
-class AsdfVersionMixin(object):
+class AsdfVersionMixin:
     """This mix-in is required in order to impose the total ordering that we
     want for ``AsdfVersion``, rather than accepting the total ordering that is
     already provided by ``Version`` from ``semantic_version``. Defining these
@@ -131,17 +158,17 @@ class AsdfSpec(SpecItem, Spec):
         return super(AsdfSpec, self).__hash__()
 
 
-default_version = AsdfVersion('1.2.0')
-
-
 supported_versions = [
     AsdfVersion('1.0.0'),
     AsdfVersion('1.1.0'),
-    AsdfVersion('1.2.0')
+    AsdfVersion('1.2.0'),
+    AsdfVersion('1.3.0')
 ]
 
+default_version = supported_versions[-1]
 
-class VersionedMixin(object):
+
+class VersionedMixin:
     _version = default_version
 
     @property
@@ -154,8 +181,9 @@ class VersionedMixin(object):
             human_versions = util.human_list(
                 [str(x) for x in supported_versions])
             raise ValueError(
-                "asdf only understands how to handle ASDF versions {0}. "
-                "Got '{1}'".format(human_versions, version))
+                "This version of the asdf package ({0}) only understands how "
+                "to handle versions {1} of the ASDF Standard. Got "
+                "'{2}'".format(asdf_version, human_versions, version))
 
         self._version = version
 
