@@ -2,7 +2,6 @@ import os
 import sys
 import stat
 import shutil
-import warnings
 import contextlib
 
 import pytest
@@ -13,11 +12,12 @@ from setuptools import Distribution
 
 from ..setup_helpers import get_package_info, register_commands
 from ..commands import build_ext
-from ..utils import AstropyDeprecationWarning
 
-from . import reset_setup_helpers, reset_distutils_log, fix_hide_setuptools  # noqa
+from . import reset_setup_helpers, reset_distutils_log  # noqa
 from . import run_setup, cleanup_import
 
+ASTROPY_HELPERS_PATH = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', '..'))
+
 
 def _extension_test_package(tmpdir, request, extension_type='c'):
     """Creates a simple test package with an extension module."""
@@ -86,12 +86,19 @@ def _extension_test_package(tmpdir, request, extension_type='c'):
     """.format(', '.join(extensions_list))))
 
     test_pkg.join('setup.py').write(dedent("""\
+        import sys
         from os.path import join
         from setuptools import setup
+        sys.path.insert(0, r'{astropy_helpers_path}')
         from astropy_helpers.setup_helpers import register_commands
         from astropy_helpers.setup_helpers import get_package_info
         from astropy_helpers.version_helpers import generate_version_py
 
+        if '--no-cython' in sys.argv:
+            from astropy_helpers.commands import build_ext
+            build_ext.should_build_with_cython = lambda *args: False
+            sys.argv.remove('--no-cython')
+
         NAME = 'apyhtest_eva'
         VERSION = '0.1'
         RELEASE = True
@@ -106,7 +113,7 @@ def _extension_test_package(tmpdir, request, extension_type='c'):
             cmdclass=cmdclassd,
             **package_info
         )
-    """))
+    """.format(astropy_helpers_path=ASTROPY_HELPERS_PATH)))
 
     if '' in sys.path:
         sys.path.remove('')
@@ -160,7 +167,7 @@ def test_cython_autoextensions(tmpdir):
     assert package_info['ext_modules'][0].name == 'yoda.luke.dagobah'
 
 
-def test_compiler_module(c_extension_test_package):
+def test_compiler_module(capsys, c_extension_test_package):
     """
     Test ensuring that the compiler module is built and installed for packages
     that have extension modules.
@@ -172,16 +179,14 @@ def test_compiler_module(c_extension_test_package):
     with test_pkg.as_cwd():
         # This is one of the simplest ways to install just a package into a
         # test directory
-        with warnings.catch_warnings(record=True) as w:
-            run_setup('setup.py',
-                      ['install',
-                       '--single-version-externally-managed',
-                       '--install-lib={0}'.format(install_temp),
-                       '--record={0}'.format(install_temp.join('record.txt'))])
+        run_setup('setup.py',
+                  ['install',
+                   '--single-version-externally-managed',
+                   '--install-lib={0}'.format(install_temp),
+                   '--record={0}'.format(install_temp.join('record.txt'))])
 
-        # Warning expected from get_git_devstr, called by generate_version_py
-        assert len(w) == 1
-        assert str(w[0].message).startswith("No git repository present at")
+        stdout, stderr = capsys.readouterr()
+        assert "No git repository present at" in stderr
 
     with install_temp.as_cwd():
         import apyhtest_eva
@@ -195,7 +200,7 @@ def test_compiler_module(c_extension_test_package):
         assert apyhtest_eva.version.compiler != 'unknown'
 
 
-def test_no_cython_buildext(c_extension_test_package, monkeypatch):
+def test_no_cython_buildext(capsys, c_extension_test_package, monkeypatch):
     """
     Regression test for https://github.com/astropy/astropy-helpers/pull/35
 
@@ -206,17 +211,12 @@ def test_no_cython_buildext(c_extension_test_package, monkeypatch):
 
     test_pkg = c_extension_test_package
 
-    # In order for this test to test the correct code path we need to fool
-    # build_ext into thinking we don't have Cython installed
-    monkeypatch.setattr(build_ext, 'should_build_with_cython',
-                        lambda *args: False)
-
     with test_pkg.as_cwd():
-        with warnings.catch_warnings(record=True) as w:
-            run_setup('setup.py', ['build_ext', '--inplace'])
 
-        assert len(w) == 1
-        assert str(w[0].message).startswith("No git repository present at")
+        run_setup('setup.py', ['build_ext', '--inplace', '--no-cython'])
+
+        stdout, stderr = capsys.readouterr()
+        assert "No git repository present at" in stderr
 
     sys.path.insert(0, str(test_pkg))
 
@@ -228,7 +228,7 @@ def test_no_cython_buildext(c_extension_test_package, monkeypatch):
         sys.path.remove(str(test_pkg))
 
 
-def test_missing_cython_c_files(pyx_extension_test_package, monkeypatch):
+def test_missing_cython_c_files(capsys, pyx_extension_test_package, monkeypatch):
     """
     Regression test for https://github.com/astropy/astropy-helpers/pull/181
 
@@ -238,35 +238,25 @@ def test_missing_cython_c_files(pyx_extension_test_package, monkeypatch):
 
     test_pkg = pyx_extension_test_package
 
-    # In order for this test to test the correct code path we need to fool
-    # build_ext into thinking we don't have Cython installed
-    monkeypatch.setattr(build_ext, 'should_build_with_cython',
-                        lambda *args: False)
-
     with test_pkg.as_cwd():
-        with pytest.raises(SystemExit) as exc_info:
-            with warnings.catch_warnings(record=True) as w:
-                run_setup('setup.py', ['build_ext', '--inplace'])
 
-            assert len(w) == 1
-            assert str(w[0].message).startswith(
-                "No git repository present at")
+        run_setup('setup.py', ['build_ext', '--inplace', '--no-cython'])
 
-    msg = ('Could not find C/C++ file '
-           '{0}.(c/cpp)'.format('apyhtest_eva/unit02'.replace('/', os.sep)))
+        stdout, stderr = capsys.readouterr()
+        assert "No git repository present at" in stderr
 
-    assert msg in str(exc_info.value)
+        msg = ('Could not find C/C++ file '
+               '{0}.(c/cpp)'.format('apyhtest_eva/unit02'.replace('/', os.sep)))
+
+        assert msg in stderr
 
 
-@pytest.mark.parametrize('mode', ['cli', 'cli-w', 'direct', 'deprecated', 'cli-l', 'cli-error'])
-def test_build_docs(tmpdir, mode):
+@pytest.mark.parametrize('mode', ['cli', 'cli-w', 'deprecated', 'cli-l', 'cli-error'])
+def test_build_docs(capsys, tmpdir, mode):
     """
     Test for build_docs
     """
 
-    import astropy_helpers
-    ah_path = os.path.dirname(astropy_helpers.__file__)
-
     test_pkg = tmpdir.mkdir('test_pkg')
 
     test_pkg.mkdir('mypackage')
@@ -282,8 +272,9 @@ def test_build_docs(tmpdir, mode):
             pass
     """))
 
-    docs = test_pkg.mkdir('docs')
+    test_pkg.mkdir('docs')
 
+    docs = test_pkg.join('docs')
     autosummary = docs.mkdir('_templates').mkdir('autosummary')
 
     autosummary.join('base.rst').write('{% extends "autosummary_core/base.rst" %}')
@@ -312,6 +303,8 @@ def test_build_docs(tmpdir, mode):
     """))
 
     test_pkg.join('setup.py').write(dedent("""\
+        import sys
+        sys.path.insert(0, r'{astropy_helpers_path}')
         from os.path import join
         from setuptools import setup, Extension
         from astropy_helpers.setup_helpers import register_commands, get_package_info
@@ -328,10 +321,9 @@ def test_build_docs(tmpdir, mode):
             cmdclass=cmdclassd,
             **get_package_info()
         )
-    """))
+    """.format(astropy_helpers_path=ASTROPY_HELPERS_PATH)))
 
     with test_pkg.as_cwd():
-        shutil.copytree(ah_path, 'astropy_helpers')
 
         if mode == 'cli':
             run_setup('setup.py', ['build_docs'])
@@ -340,15 +332,9 @@ def test_build_docs(tmpdir, mode):
         elif mode == 'cli-l':
             run_setup('setup.py', ['build_docs', '-l'])
         elif mode == 'deprecated':
-            with pytest.warns(AstropyDeprecationWarning):
-                run_setup('setup.py', ['build_sphinx'])
-        elif mode == 'direct':  # to check coverage
-            with docs_dir.as_cwd():
-                from sphinx import main
-                try:
-                    main(['-b html', '-d _build/doctrees', '.', '_build/html'])
-                except SystemExit as exc:
-                    assert exc.code == 0
+            run_setup('setup.py', ['build_sphinx'])
+            stdout, stderr = capsys.readouterr()
+            assert 'AstropyDeprecationWarning' in stderr
 
 
 def test_command_hooks(tmpdir, capsys):
@@ -371,8 +357,10 @@ def test_command_hooks(tmpdir, capsys):
     # A simple setup.py for the test package--running register_commands should
     # discover and enable the command hooks
     test_pkg.join('setup.py').write(dedent("""\
+        import sys
         from os.path import join
         from setuptools import setup, Extension
+        sys.path.insert(0, r'{astropy_helpers_path}')
         from astropy_helpers.setup_helpers import register_commands, get_package_info
 
         NAME = '_welltall_'
@@ -386,7 +374,7 @@ def test_command_hooks(tmpdir, capsys):
             version=VERSION,
             cmdclass=cmdclassd
         )
-    """))
+    """.format(astropy_helpers_path=ASTROPY_HELPERS_PATH)))
 
     with test_pkg.as_cwd():
         try:
@@ -403,7 +391,7 @@ def test_command_hooks(tmpdir, capsys):
         Goodbye build!
     """).strip()
 
-    assert want in stdout
+    assert want in stdout.replace('\r\n', '\n').replace('\r', '\n')
 
 
 def test_adjust_compiler(monkeypatch, tmpdir):
@@ -541,8 +529,10 @@ def test_invalid_package_exclusion(tmpdir, capsys):
 
     module_name = 'foobar'
     setup_header = dedent("""\
+        import sys
         from os.path import join
         from setuptools import setup, Extension
+        sys.path.insert(0, r'{astropy_helpers_path}')
         from astropy_helpers.setup_helpers import register_commands, \\
             get_package_info, add_exclude_packages
 
@@ -550,7 +540,7 @@ def test_invalid_package_exclusion(tmpdir, capsys):
         VERSION = 0.1
         RELEASE = True
 
-    """.format(module_name=module_name))
+    """.format(module_name=module_name, astropy_helpers_path=ASTROPY_HELPERS_PATH))
 
     setup_footer = dedent("""\
         setup(
@@ -574,14 +564,13 @@ def test_invalid_package_exclusion(tmpdir, capsys):
         setup_header + error_commands + setup_footer)
 
     with error_pkg.as_cwd():
-        try:
-            with pytest.raises(RuntimeError):
-                run_setup('setup.py', ['build'])
-        finally:
-            cleanup_import(module_name)
+        run_setup('setup.py', ['build'])
+
+        stdout, stderr = capsys.readouterr()
+        assert "RuntimeError" in stderr
 
     # Test warning when using deprecated exclude parameter
-    warn_commands =  dedent("""\
+    warn_commands = dedent("""\
         cmdclassd = register_commands(NAME, VERSION, RELEASE)
         package_info = get_package_info(exclude=['test*'])
 
@@ -592,8 +581,6 @@ def test_invalid_package_exclusion(tmpdir, capsys):
         setup_header + warn_commands + setup_footer)
 
     with warn_pkg.as_cwd():
-        try:
-            with pytest.warns(AstropyDeprecationWarning):
-                run_setup('setup.py', ['build'])
-        finally:
-            cleanup_import(module_name)
+        run_setup('setup.py', ['build'])
+        stdout, stderr = capsys.readouterr()
+        assert 'AstropyDeprecationWarning' in stderr
