@@ -4,11 +4,12 @@
 import ah_bootstrap
 import pkg_resources
 from setuptools import setup
-from astropy_helpers.setup_helpers import register_commands, get_package_info
+from astropy_helpers.setup_helpers import (register_commands, get_package_info,
+                                           add_exclude_packages)
 from astropy_helpers.version_helpers import generate_version_py
 
 NAME = 'astropy_helpers'
-VERSION = '3.0.dev'
+VERSION = '3.0.2'
 RELEASE = 'dev' not in VERSION
 
 generate_version_py(NAME, VERSION, RELEASE, False, uses_git=not RELEASE)
@@ -16,6 +17,7 @@ generate_version_py(NAME, VERSION, RELEASE, False, uses_git=not RELEASE)
 # Use the updated version including the git rev count
 from astropy_helpers.version import version as VERSION
 
+add_exclude_packages(['astropy_helpers.tests'])
 cmdclass = register_commands(NAME, VERSION, RELEASE)
 # This package actually doesn't use the Astropy test command
 del cmdclass['test']
@@ -47,5 +49,5 @@ setup(
     cmdclass=cmdclass,
     python_requires='>=3.5',
     zip_safe=False,
-    **get_package_info(exclude=['astropy_helpers.tests'])
+    **get_package_info()
 )
